/*----------------------------------------------------------------------------
    ChucK Concurrent, On-the-fly Audio Programming Language
      Compiler and Virtual Machine

    Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
      http://chuck.cs.princeton.edu/
      http://soundlab.cs.princeton.edu/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// file: ugen_osc.cpp
// desc: ...
//
// author: Ge Wang (gewang@cs.princeton.edu)
//         Perry R. Cook (prc@cs.princeton.edu)
// date: Spring 2004
//-----------------------------------------------------------------------------
#include "ugen_osc.h"
#include "chuck_bbq.h"
#include <math.h>
#include <stdio.h>


//-----------------------------------------------------------------------------
// name: osc_query()
// desc: ...
//-----------------------------------------------------------------------------
DLL_QUERY osc_query( Chuck_DL_Query * QUERY )
{
    // add sinosc
    QUERY->ugen_add( QUERY, "sinosc", NULL );
    // set funcs
    QUERY->ugen_func( QUERY, sinosc_ctor, sinosc_dtor, sinosc_tick, sinosc_pmsg );
    // add ctrl
    QUERY->ugen_ctrl( QUERY, sinosc_ctrl_freq, "float", "freq" );

    return TRUE;
}




//-----------------------------------------------------------------------------
// name: struct Osc_Data
// desc: ...
//-----------------------------------------------------------------------------
struct Osc_Data
{
    double t;
    double num;
    double freq;
    t_CKUINT srate;
    
    Osc_Data( )
    {
        t = 0.0;
        num = 0.0;
        freq = 220.0;
        srate = Digitalio::sampling_rate();
        float f = (float)freq;
        sinosc_ctrl_freq( 0, this, &f );
    }
};


//-----------------------------------------------------------------------------
// name: sinosc_ctor()
// desc: ...
//-----------------------------------------------------------------------------
UGEN_CTOR sinosc_ctor( t_CKTIME now )
{
    // return data to be used later
    return new Osc_Data;
}




//-----------------------------------------------------------------------------
// name: sinosc_dtor()
// desc: ...
//-----------------------------------------------------------------------------
UGEN_DTOR sinosc_dtor( t_CKTIME now, void * data )
{
    // delete
    delete (Osc_Data *)data;
}




//-----------------------------------------------------------------------------
// name: sinosc_tick()
// desc: ...
//-----------------------------------------------------------------------------
UGEN_TICK sinosc_tick( t_CKTIME now, void * data, SAMPLE in, SAMPLE * out )
{
    Osc_Data * d = (Osc_Data *)data;
    *out = (SAMPLE)sin( d->t * d->num );
    d->t += 1.0;

    return TRUE;
}




//-----------------------------------------------------------------------------
// name: sinosc_ctrl_freq()
// desc: ...
//-----------------------------------------------------------------------------
UGEN_CTRL sinosc_ctrl_freq( t_CKTIME now, void * data, void * value )
{
    Osc_Data * d = (Osc_Data *)data;
    d->freq = (float)GET_CK_FLOAT(value);
    d->num = 2.0 * 3.14159265358979323846 * d->freq / d->srate;
}




//-----------------------------------------------------------------------------
// name: sinosc_pmsg()
// desc: ...
//-----------------------------------------------------------------------------
UGEN_PMSG sinosc_pmsg( t_CKTIME now, void * data, const char * msg, void * value )
{
    Osc_Data * d = (Osc_Data *)data;
    if( !strcmp( msg, "print" ) )
    {
        fprintf( stdout, "sinosc: (freq=%f)", d->freq );
        return TRUE;
    }
    
    // didn't handle
    return FALSE;
}
